// Package main provides a simple tool to check if a list of OpenAI keys are valid.
package main

import (
	"bufio"
	"fmt"
	"net/http"
	"os"
	"sync"
	"time"

	ct "github.com/daviddengcn/go-colortext"
)

const openaiEndpoint = "https://api.openai.com/v1/engines/davinci-codex/completions"

var (
	valid   = 0
	invalid = 0
	total   = 0
	date    string
)

// main is the entrypoint for the program.
func main() {
	keysFile, err := os.Open(os.Args[1])
	if err != nil {
		fmt.Println("Error opening keys file:", err)
		return
	}
	defer keysFile.Close()

	var wg sync.WaitGroup
	scanner := bufio.NewScanner(keysFile)
	for scanner.Scan() {
		date = time.Now().Format("15:04:05")
		total++
		key := scanner.Text()
		wg.Add(1)
		go func(k string) {
			defer wg.Done()
			client := &http.Client{}
			req, err := http.NewRequest("POST", openaiEndpoint, nil)
			if err != nil {
				fmt.Println("Error creating request:", err)
				return
			}
			req.Header.Set("Authorization", "Bearer "+k)
			res, err := client.Do(req)
			if err != nil || res.StatusCode == 401 {
				invalid++
				ct.ChangeColor(ct.Red, false, ct.None, false)
				fmt.Printf("[%d] [%s] [%d] [INVALID]: %s\n", total, date, valid, k)
				return
			}
			valid++
			ct.ChangeColor(ct.Green, false, ct.None, false)
			fmt.Printf("[%d] [%s] [%d] [VALID]: %s\n", total, date, valid, k)
		}(key)
	}
	wg.Wait()

	ct.ChangeColor(ct.Blue, false, ct.None, false)
	fmt.Printf("[%d] [%s] [COMPLETE]: Valid: [%d] - Invalid: [%d]\n", total, date, valid, invalid)
}
